
#ifndef KF5GLOBALACCELPRIVATE_EXPORT_H
#define KF5GLOBALACCELPRIVATE_EXPORT_H

#ifdef KF5GLOBALACCELPRIVATE_STATIC_DEFINE
#  define KF5GLOBALACCELPRIVATE_EXPORT
#  define KF5GLOBALACCELPRIVATE_NO_EXPORT
#else
#  ifndef KF5GLOBALACCELPRIVATE_EXPORT
#    ifdef KF5GlobalAccelPrivate_EXPORTS
        /* We are building this library */
#      define KF5GLOBALACCELPRIVATE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define KF5GLOBALACCELPRIVATE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef KF5GLOBALACCELPRIVATE_NO_EXPORT
#    define KF5GLOBALACCELPRIVATE_NO_EXPORT 
#  endif
#endif

#ifndef KF5GLOBALACCELPRIVATE_DEPRECATED
#  define KF5GLOBALACCELPRIVATE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KF5GLOBALACCELPRIVATE_DEPRECATED_EXPORT
#  define KF5GLOBALACCELPRIVATE_DEPRECATED_EXPORT KF5GLOBALACCELPRIVATE_EXPORT KF5GLOBALACCELPRIVATE_DEPRECATED
#endif

#ifndef KF5GLOBALACCELPRIVATE_DEPRECATED_NO_EXPORT
#  define KF5GLOBALACCELPRIVATE_DEPRECATED_NO_EXPORT KF5GLOBALACCELPRIVATE_NO_EXPORT KF5GLOBALACCELPRIVATE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KF5GLOBALACCELPRIVATE_NO_DEPRECATED
#    define KF5GLOBALACCELPRIVATE_NO_DEPRECATED
#  endif
#endif

#endif
